@@ -209,11 +209,8 @@ gegl_path_class_init (GeglPathClass *klass)
   gegl_path_signals[GEGL_PATH_CHANGED] =
     g_signal_new ("changed", G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
-                  0    /* class offset */,
-                  NULL /* accumulator */,
-                  NULL /* accu_data */,
-                  g_cclosure_marshal_VOID__POINTER,
-                  G_TYPE_NONE, /*return type */
+                  0, NULL, NULL, NULL,
+                  G_TYPE_NONE,
                   1, G_TYPE_POINTER);
 }
 
@@ -997,7 +994,8 @@ gegl_path_add_flattener (GeglPathList *(*flattener) (GeglPathList *original))
       if (klass->flattener[i]==NULL)
         {
           klass->flattener[i] = flattener;
-          klass->flattener[i+1] = NULL;
+          if (i + 1 < 8)
+            klass->flattener[i+1] = NULL;
           return;
         }
     }
@@ -1027,11 +1025,11 @@ parse_float_pair (const gchar *p,
 {
   gchar *t = (void*) p;
   while (*t && ((*t<'0' || *t > '9') && *t!='-')) t++;
-  if (!t)
+  if (!*t)
     return p;
   *x = g_ascii_strtod (t, &t);
   while (*t && ((*t<'0' || *t > '9') && *t!='-')) t++;
-  if (!t)
+  if (!*t)
     return p;
   *y = g_ascii_strtod (t, &t);
   return t;
@@ -1126,7 +1124,7 @@ ensure_flattened (GeglPath *vector)
   if (priv->flat_path)
     old_path = priv->flat_path;
 
-  for (i=0;klass->flattener[i];i++)
+  for (i=0;klass->flattener[i] && i < 8;i++)
     {
       new_path = klass->flattener[i] (path);
       if (new_path != path)
