@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * TMO:
  * Copyright 2010      Danny Robson      <danny@blubinc.net>
@@ -25,31 +25,33 @@
 
 #include "config.h"
 #include <glib/gi18n-lib.h>
-#include <math.h>
 
+#ifdef GEGL_PROPERTIES
 
-#ifdef GEGL_CHANT_PROPERTIES
+property_double (alpha, _("Alpha"), 1.0)
+    description(_("Gradient threshold for detail enhancement"))
+    value_range (0.0, 2.0)
 
-gegl_chant_double (alpha, _("Alpha"),
-                  0.0, 2.0, 1.0,
-                  _("Gradient threshold for detail enhancement"))
-gegl_chant_double (beta, _("Beta"),
-                  0.1, 2.0, 0.9,
-                  _("Strength of local detail enhancement"))
-gegl_chant_double (saturation, _("Saturation"),
-                  0.0, 1.0, 0.8,
-                  _("Global color saturation factor"))
-gegl_chant_double (noise, _("Noise"),
-                  0.0, 1.0, 0.0,
-                  _("Gradient threshold for lowering detail enhancement"))
+property_double (beta, _("Beta"), 0.9)
+    description(_("Strength of local detail enhancement"))
+    value_range (0.1, 2.0)
+
+property_double (saturation, _("Saturation"), 0.8)
+    description (_("Global color saturation factor"))
+    value_range (0.0, 1.0)
+
+property_double (noise, _("Noise"), 0.0)
+    description (_("Gradient threshold for lowering detail enhancement"))
+    value_range (0.0, 1.0)
 
 
 #else
 
-#define GEGL_CHANT_TYPE_FILTER
-#define GEGL_CHANT_C_FILE       "fattal02.c"
+#define GEGL_OP_FILTER
+#define GEGL_OP_NAME     fattal02
+#define GEGL_OP_C_SOURCE fattal02.c
 
-#include "gegl-chant.h"
+#include "gegl-op.h"
 #include "gegl-debug.h"
 #include <stdlib.h>
 
@@ -1028,9 +1030,11 @@ fattal02_float_cmp (const void *_a,
   const gfloat a = *(gfloat *)_a,
                b = *(gfloat *)_b;
 
-  if (a < b) return -1;
-  if (a > b) return  1;
-             return  0;
+  if (a < b)
+    return -1;
+  if (a > b)
+    return  1;
+  return  0;
 }
 
 
@@ -1215,30 +1219,32 @@ fattal02_tonemap (const gfloat        *input,   /* Y */
 static void
 fattal02_prepare (GeglOperation *operation)
 {
-  gegl_operation_set_format (operation, "input",  babl_format (OUTPUT_FORMAT));
-  gegl_operation_set_format (operation, "output", babl_format (OUTPUT_FORMAT));
+  const Babl *space = gegl_operation_get_source_space (operation, "input");
+  gegl_operation_set_format (operation, "input",  babl_format_with_space (OUTPUT_FORMAT, space));
+  gegl_operation_set_format (operation, "output", babl_format_with_space (OUTPUT_FORMAT, space));
 }
 
+static GeglRectangle
+fattal02_get_cached_region (GeglOperation       *operation,
+                            const GeglRectangle *roi)
+{
+  const GeglRectangle *in_rect =
+      gegl_operation_source_get_bounding_box (operation, "input");
+
+  if (in_rect && ! gegl_rectangle_is_infinite_plane (in_rect))
+    return *in_rect;
+
+  return *roi;
+}
 
 static GeglRectangle
 fattal02_get_required_for_output (GeglOperation       *operation,
                                   const gchar         *input_pad,
                                   const GeglRectangle *roi)
 {
-  GeglRectangle result = *gegl_operation_source_get_bounding_box (operation,
-                                                                  "input");
-  return result;
+  return fattal02_get_cached_region (operation, roi);
 }
 
-
-static GeglRectangle
-fattal02_get_cached_region (GeglOperation       *operation,
-                            const GeglRectangle *roi)
-{
-  return *gegl_operation_source_get_bounding_box (operation, "input");
-}
-
-
 static gboolean
 fattal02_process (GeglOperation       *operation,
                   GeglBuffer          *input,
@@ -1246,8 +1252,10 @@ fattal02_process (GeglOperation       *operation,
                   const GeglRectangle *result,
                   gint                 level)
 {
-  const GeglChantO *o     = GEGL_CHANT_PROPERTIES (operation);
+  const GeglProperties *o     = GEGL_PROPERTIES (operation);
   gfloat            noise;
+  const Babl *out_format = gegl_operation_get_format (operation, "output");
+  const Babl *space = babl_format_get_space (out_format);
 
   const gint  pix_stride = 3; /* RGBA */
   gfloat     *lum_in,
@@ -1260,7 +1268,7 @@ fattal02_process (GeglOperation       *operation,
   g_return_val_if_fail (output, FALSE);
   g_return_val_if_fail (result, FALSE);
 
-  g_return_val_if_fail (babl_format_get_n_components (babl_format (OUTPUT_FORMAT)) == pix_stride, FALSE);
+  g_return_val_if_fail (babl_format_get_n_components (out_format) == pix_stride, FALSE);
 
   /* Adjust noise floor if not set by the user */
   if (o->noise == 0.0)
@@ -1276,11 +1284,11 @@ fattal02_process (GeglOperation       *operation,
   lum_in  = g_new (gfloat, result->width * result->height);
   lum_out = g_new (gfloat, result->width * result->height);
 
-  gegl_buffer_get (input, result, 1.0, babl_format ("Y float"),
+  gegl_buffer_get (input, result, 1.0, babl_format_with_space ("Y float", space),
                    lum_in, GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   pix = g_new (gfloat, result->width * result->height * pix_stride);
-  gegl_buffer_get (input, result, 1.0, babl_format (OUTPUT_FORMAT),
+  gegl_buffer_get (input, result, 1.0, out_format,
                    pix, GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   fattal02_tonemap (lum_in, result, lum_out, o->alpha, o->beta, noise);
@@ -1292,7 +1300,7 @@ fattal02_process (GeglOperation       *operation,
                 lum_out[i / pix_stride]);
     }
 
-  gegl_buffer_set (output, result, 0, babl_format (OUTPUT_FORMAT), pix,
+  gegl_buffer_set (output, result, 0, out_format, pix,
                    GEGL_AUTO_ROWSTRIDE);
   g_free (pix);
   g_free (lum_out);
@@ -1300,11 +1308,34 @@ fattal02_process (GeglOperation       *operation,
   return TRUE;
 }
 
+static gboolean
+fattal02_operation_process (GeglOperation        *operation,
+                            GeglOperationContext *context,
+                            const gchar          *output_prop,
+                            const GeglRectangle  *result,
+                            gint                  level)
+{
+  GeglOperationClass  *operation_class;
+
+  const GeglRectangle *in_rect =
+    gegl_operation_source_get_bounding_box (operation, "input");
+
+  if (in_rect && gegl_rectangle_is_infinite_plane (in_rect))
+    {
+      gpointer in = gegl_operation_context_get_object (context, "input");
+      gegl_operation_context_take_object (context, "output",
+                                          g_object_ref (G_OBJECT (in)));
+      return TRUE;
+    }
+
+  operation_class = GEGL_OPERATION_CLASS (gegl_op_parent_class);
+
+  return operation_class->process (operation, context, output_prop, result,
+                                   gegl_operation_context_get_level (context));
+}
 
-/*
- */
 static void
-gegl_chant_class_init (GeglChantClass *klass)
+gegl_op_class_init (GeglOpClass *klass)
 {
   GeglOperationClass       *operation_class;
   GeglOperationFilterClass *filter_class;
@@ -1315,17 +1346,21 @@ gegl_chant_class_init (GeglChantClass *klass)
   filter_class->process = fattal02_process;
 
   operation_class->prepare                 = fattal02_prepare;
+  operation_class->process                 = fattal02_operation_process;
   operation_class->get_required_for_output = fattal02_get_required_for_output;
   operation_class->get_cached_region       = fattal02_get_cached_region;
+  operation_class->threaded                = FALSE;
 
   gegl_operation_class_set_keys (operation_class,
   "name"       , "gegl:fattal02",
-  "categories" , "tonemapping",
+  "title",       _("Fattal et al. 2002 Tone Mapping"),
+  "categories" , "tonemapping:enhance",
+  "reference-hash", "6c025821070a93a3feae9fe2e563b9d0",
   "description",
         _("Adapt an image, which may have a high dynamic range, for "
 	  "presentation using a low dynamic range. This operator attenuates "
           "the magnitudes of local image gradients, producing luminance "
-          "within the range 0.0-1.0"),
+          "within the range 0.0-1.0. This tonemapping approach was originally presented by Raanan Fattal, in the 2002 SIGGRAPH paper: Gradient Domain High Dynamic Range Compression."),
         NULL);
 }
 
