@@ -17,7 +17,6 @@
 
 #include "config.h"
 #include <glib/gi18n-lib.h>
-#include <math.h>
 
 #ifdef GEGL_PROPERTIES
 
@@ -99,8 +98,6 @@ attach (GeglOperation *operation)
   gegl_operation_meta_redirect (operation, "abyss-policy", vblur, "abyss-policy");
   gegl_operation_meta_redirect (operation, "filter",       vblur, "filter");
   gegl_operation_meta_redirect (operation, "clip-extent",  vblur, "clip-extent");
-
-  gegl_operation_meta_watch_nodes (operation, hblur, vblur, NULL);
 }
 
 static void
