@@ -22,9 +22,11 @@
 #ifdef GEGL_PROPERTIES
 
 enum_start (gegl_vignette_shape)
-  enum_value (GEGL_VIGNETTE_SHAPE_CIRCLE,  "circle",  N_("Circle"))
-  enum_value (GEGL_VIGNETTE_SHAPE_SQUARE,  "square",  N_("Square"))
-  enum_value (GEGL_VIGNETTE_SHAPE_DIAMOND, "diamond", N_("Diamond"))
+  enum_value (GEGL_VIGNETTE_SHAPE_CIRCLE,     "circle",     N_("Circle"))
+  enum_value (GEGL_VIGNETTE_SHAPE_SQUARE,     "square",     N_("Square"))
+  enum_value (GEGL_VIGNETTE_SHAPE_DIAMOND,    "diamond",    N_("Diamond"))
+  enum_value (GEGL_VIGNETTE_SHAPE_HORIZONTAL, "horizontal", N_("Horizontal"))
+  enum_value (GEGL_VIGNETTE_SHAPE_VERTICAL,   "vertical",   N_("Vertical"))
 enum_end (GeglVignetteShape)
 
 property_enum (shape, _("Vignette shape"),
@@ -37,7 +39,8 @@ property_color (color, _("Color"), "black")
 
 property_double (radius, _("Radius"), 1.2)
     description (_("How far out vignetting goes as portion of half image diagonal"))
-    value_range (0.0, 3.0)
+    value_range (0.0, G_MAXDOUBLE)
+    ui_range    (0.0, 3.0)
     ui_meta     ("unit", "relative-distance")
 
 property_double (softness, _("Softness"), 0.8)
@@ -45,7 +48,8 @@ property_double (softness, _("Softness"), 0.8)
 
 property_double (gamma, _("Gamma"), 2.0)
     description (_("Falloff linearity"))
-    value_range (0.0, 10.0)
+    value_range (0.0, G_MAXDOUBLE)
+    ui_range    (0.0, 10.0)
 
 property_double (proportion, _("Proportion"), 1.0)
     description(_("How close we are to image proportions"))
@@ -81,8 +85,6 @@ property_double (rotation, _("Rotation"), 0.0)
 
 #include "gegl-op.h"
 
-#include <math.h>
-
 static void
 prepare (GeglOperation *operation)
 {
@@ -190,7 +192,7 @@ cl_process (GeglOperation       *operation,
   const size_t gbl_size[2] = {roi->width, roi->height};
 
   gint   shape = (gint) o->shape;
-  gfloat gamma = o->gamma;
+  gfloat gamma = MAX (o->gamma, 0.0001);
 
   cl_int cl_err = 0;
   cl_float4 f_color;
@@ -261,6 +263,7 @@ process (GeglOperation       *operation,
   GeglRectangle *bounds = gegl_operation_source_get_bounding_box (operation, "input");
   gfloat length = hypot (bounds->width, bounds->height)/2;
   gfloat rdiff;
+  gfloat gamma;
   gfloat cost, sint;
   gfloat costy, sinty;
 
@@ -287,6 +290,8 @@ process (GeglOperation       *operation,
   if (fabs (rdiff) < 0.0001)
     rdiff = 0.0001;
 
+  gamma = MAX (o->gamma, 0.0001);
+
   midx = bounds->x + bounds->width * o->x;
   midy = bounds->y + bounds->height * o->y;
 
@@ -325,6 +330,10 @@ process (GeglOperation       *operation,
               strength = MAX(ABS(u-midx) / scale, ABS(v-midy)); break;
             case GEGL_VIGNETTE_SHAPE_DIAMOND:
               strength = ABS(u-midx) / scale + ABS(v-midy);     break;
+            case GEGL_VIGNETTE_SHAPE_HORIZONTAL:
+              strength = ABS(v-midy);                           break;
+            case GEGL_VIGNETTE_SHAPE_VERTICAL:
+              strength = ABS(u-midx) / scale;                   break;
           }
           strength /= length;
           strength = (strength-radius0) / rdiff;
@@ -335,13 +344,13 @@ process (GeglOperation       *operation,
       if (strength>1.0)
         strength = 1.0;
 
-      if (o->gamma > 1.9999 && o->gamma < 2.0001)
+      if (gamma > 1.9999 && gamma < 2.0001)
         strength *= strength;  /* fast path for default gamma */
-      else if (o->gamma != 1.0)
-        strength = powf(strength, o->gamma); /* this gamma factor is
-                                              * very expensive.. and precision
-                                              * is not very important
-                                              */
+      else if (gamma != 1.0)
+        strength = powf(strength, gamma); /* this gamma factor is
+                                           * very expensive.. and precision
+                                           * is not very important
+                                           */
 
       out_pixel[0]=in_pixel[0] * (1.0-strength) + color[0] * strength;
       out_pixel[1]=in_pixel[1] * (1.0-strength) + color[1] * strength;
