@@ -14,7 +14,7 @@ TEST ()
     gfloat *buf;
     gint    x, y, i;
 
-    buf = (gpointer)gegl_buffer_linear_open (buffer, &extent, &rowstride, NULL);
+    buf = gegl_buffer_linear_open (buffer, &extent, &rowstride, NULL);
     g_assert (buf);
 
     i=0;
@@ -28,6 +28,6 @@ TEST ()
   fill_rect (buffer, &roi, 0.2);
 
   print_buffer (buffer);
+  g_object_unref (buffer);
   test_end ();
-  gegl_buffer_destroy (buffer);
 }
